/*
**	SerialPanel.c
**
**	Editing panel for serial configuration
**
**	Copyright  1990-1993 by Olaf `Olsen' Barthel & MXM
**		All Rights Reserved
*/

#include "termGlobal.h"

enum	{	GAD_BAUD,GAD_DATABITS,GAD_PARITY,GAD_STOPBITS,GAD_HANDSHAKING,GAD_DUPLEX,GAD_BUFFERSIZE,GAD_BREAKLENGTH,
		GAD_STRIP,GAD_HIGHSPEED,GAD_CHECK_CARRIER,GAD_SHARED,GAD_XON_XOFF,GAD_PASSTHROUGH,
		GAD_DEVICE,GAD_SELECT,GAD_UNIT,GAD_USE,GAD_CANCEL,GAD_DEFAULT };

STATIC LONG Sizes[11] =
{
	512,1024,2048,4096,8192,16384,32768,65536,131072,262144,524288
};

STATIC STRPTR	DataBits[3],
		Parity[6],
		StopBits[3],
		Handshaking[4],
		Duplex[3];

STATIC LONG __saveds
ShowBaud(struct Gadget *SomeGadget,WORD Level)
{
	return(BaudRates[Level]);
}

STATIC LONG __saveds
ShowSize(struct Gadget *SomeGadget,WORD Level)
{
	return(Sizes[Level]);
}

STATIC struct Gadget *
CreateAllGadgets(struct Configuration *Config,struct Gadget **GadgetArray,struct Gadget **GadgetList,BYTE *Default)
{
	struct Gadget		*Gadget;
	struct NewGadget	 NewGadget;
	UWORD			 Counter = 0;
	WORD			 i,Rate,Size;

	SZ_SizeSetup(Window -> WScreen,&UserFont,TRUE);

	LocalizeString(DataBits,MSG_SERIALPANEL_SEVEN_BITS_TXT,MSG_SERIALPANEL_EIGHT_BITS_TXT);
	LocalizeString(Parity,MSG_SERIALPANEL_NO_PARITY_TXT,MSG_SERIALPANEL_SPACE_TXT);
	LocalizeString(StopBits,MSG_SERIALPANEL_ONE_BIT_TXT,MSG_SERIALPANEL_TWO_BITS_TXT);
	LocalizeString(Handshaking,MSG_SERIALPANEL_HANDSHAKING_NONE_TXT,MSG_SERIALPANEL_HANDSHAKING_RTS_CTS_DSR_TXT);
	LocalizeString(Duplex,MSG_SERIALPANEL_DUPLEX_FULL_TXT,MSG_SERIALPANEL_DUPLEX_HALF_TXT);

	memset(&NewGadget,0,sizeof(struct NewGadget));

	if(Gadget = CreateContext(GadgetList))
	{
		for(i = 0 ; i < NumBaudRates ; i++)
		{
			if(Config -> SerialConfig -> BaudRate == BaudRates[i])
			{
				Rate = i;

				break;
			}
		}

		for(i = 0 ; i < sizeof(Sizes) / sizeof(LONG) ; i++)
		{
			if(Config -> SerialConfig -> SerialBufferSize == Sizes[i])
			{
				Size = i;

				break;
			}
		}

		SZ_AddLeftOffset(SZ_LeftOffsetDelta(MSG_SERIALPANEL_BAUD_RATE_GAD,MSG_SERIALPANEL_BREAK_LENGTH_GAD));

		SZ_ResetMaxWidth();

		SZ_UpdateMaxWidth(CYCLE_KIND,NULL,0,DataBits);
		SZ_UpdateMaxWidth(CYCLE_KIND,NULL,0,Parity);
		SZ_UpdateMaxWidth(CYCLE_KIND,NULL,0,StopBits);
		SZ_UpdateMaxWidth(CYCLE_KIND,NULL,0,Handshaking);
		SZ_UpdateMaxWidth(CYCLE_KIND,NULL,0,Duplex);
		SZ_UpdateMaxWidth(INTEGER_KIND,NULL,8,NULL);

		SZ_SetWidth(SZ_ResetMaxWidth());

			/* Are we in high speed mode? If so, reset
			 * the parameters to defaults.
			 */

		if(Config -> SerialConfig -> HighSpeed)
		{
			Config -> SerialConfig -> BitsPerChar	= 8;
			Config -> SerialConfig -> StopBits	= 1;
			Config -> SerialConfig -> Parity		= PARITY_NONE;
		}

		NewGadget . ng_GadgetText	= LocaleString(MSG_SERIALPANEL_BAUD_RATE_GAD);
		NewGadget . ng_GadgetID		= Counter;

		GadgetArray[Counter++] = Gadget = CreateGadget(SLIDER_KIND,Gadget,&NewGadget,
			SZ_Adjust,		TRUE,
			SZ_AutoWidth,		TRUE,

			GTSL_Min,		0,
			GTSL_Max,		NumBaudRates - 1,
			GTSL_Level,		Rate,
			GTSL_DispFunc,		ShowBaud,
			GTSL_LevelFormat,	LocaleBase ? "%7lD" : "%7ld",
			GTSL_MaxLevelLen,	7,
		TAG_DONE);

		NewGadget . ng_GadgetText	= LocaleString(MSG_SERIALPANEL_BITS_PER_CHAR_GAD);
		NewGadget . ng_GadgetID		= Counter;

		GadgetArray[Counter++] = Gadget = CreateGadget(CYCLE_KIND,Gadget,&NewGadget,
			SZ_Adjust,	TRUE,
			SZ_AutoWidth,	TRUE,

			GA_Disabled,	Config -> SerialConfig -> HighSpeed,
			GTCY_Labels,	DataBits,
			GTCY_Active,	Config -> SerialConfig -> BitsPerChar - 7,
		TAG_DONE);

		NewGadget . ng_GadgetText	= LocaleString(MSG_SERIALPANEL_PARITY_GAD);
		NewGadget . ng_GadgetID		= Counter;

		GadgetArray[Counter++] = Gadget = CreateGadget(CYCLE_KIND,Gadget,&NewGadget,
			SZ_Adjust,	TRUE,
			SZ_AutoWidth,	TRUE,

			GA_Disabled,	Config -> SerialConfig -> HighSpeed,
			GTCY_Labels,	Parity,
			GTCY_Active,	Config -> SerialConfig -> Parity,
		TAG_DONE);

		NewGadget . ng_GadgetText	= LocaleString(MSG_SERIALPANEL_STOP_BITS_GAD);
		NewGadget . ng_GadgetID		= Counter;

		GadgetArray[Counter++] = Gadget = CreateGadget(CYCLE_KIND,Gadget,&NewGadget,
			SZ_Adjust,	TRUE,
			SZ_AutoWidth,	TRUE,

			GA_Disabled,	Config -> SerialConfig -> HighSpeed,
			GTCY_Labels,	StopBits,
			GTCY_Active,	Config -> SerialConfig -> StopBits - 1,
		TAG_DONE);

		NewGadget . ng_GadgetText	= LocaleString(MSG_SERIALPANEL_HANDSHAKING_GAD);
		NewGadget . ng_GadgetID		= Counter;

		GadgetArray[Counter++] = Gadget = CreateGadget(CYCLE_KIND,Gadget,&NewGadget,
			SZ_Adjust,	TRUE,
			SZ_AutoWidth,	TRUE,

			GTCY_Labels,	Handshaking,
			GTCY_Active,	Config -> SerialConfig -> HandshakingProtocol,
		TAG_DONE);

		NewGadget . ng_GadgetText	= LocaleString(MSG_SERIALPANEL_DUPLEX_GAD);
		NewGadget . ng_GadgetID		= Counter;

		GadgetArray[Counter++] = Gadget = CreateGadget(CYCLE_KIND,Gadget,&NewGadget,
			SZ_Adjust,	TRUE,
			SZ_AutoWidth,	TRUE,

			GTCY_Labels,	Duplex,
			GTCY_Active,	Config -> SerialConfig -> Duplex,
		TAG_DONE);

		NewGadget . ng_GadgetText	= LocaleString(MSG_SERIALPANEL_BUFFER_SIZE_GAD);
		NewGadget . ng_GadgetID		= Counter;

		GadgetArray[Counter++] = Gadget = CreateGadget(SLIDER_KIND,Gadget,&NewGadget,
			SZ_Adjust,		TRUE,
			SZ_AutoWidth,		TRUE,

			GTSL_Min,		0,
			GTSL_Max,		10,
			GTSL_Level,		Size,
			GTSL_DispFunc,		ShowSize,
			GTSL_LevelFormat,	LocaleBase ? "%7lD" : "%7ld",
			GTSL_MaxLevelLen,	7,
		TAG_DONE);

		NewGadget . ng_GadgetText	= LocaleString(MSG_SERIALPANEL_BREAK_LENGTH_GAD);
		NewGadget . ng_GadgetID		= Counter;

		GadgetArray[Counter++] = Gadget = CreateGadget(INTEGER_KIND,Gadget,&NewGadget,
			SZ_Adjust,	TRUE,
			SZ_AutoWidth,	TRUE,

			GTIN_MaxChars,	8,
			GTIN_Number,	Config -> SerialConfig -> BreakLength,
		TAG_DONE);

		SZ_AddLeftOffset(SZ_LeftOffsetDelta(MSG_SERIALPANEL_STRIP_BIT_8_GAD,MSG_SERIALPANEL_DEVICE_UNIT_NUMBER_GAD));

		SZ_UpdateMaxWidth(FOLDER_KIND,NULL,15,NULL);
		SZ_UpdateMaxWidth(INTEGER_KIND,NULL,15,NULL);

		SZ_SetWidth(SZ_ResetMaxWidth());

		NewGadget . ng_GadgetText	= LocaleString(MSG_SERIALPANEL_STRIP_BIT_8_GAD);
		NewGadget . ng_GadgetID		= Counter;

		GadgetArray[Counter++] = Gadget = CreateGadget(CHECKBOX_KIND,Gadget,&NewGadget,
			SZ_Adjust,	TRUE,
			SZ_NewColumn,	TRUE,

			GTCB_Checked,	Config -> SerialConfig -> StripBit8,
		TAG_DONE);

		NewGadget . ng_GadgetText	= LocaleString(MSG_SERIALPANEL_HIGH_SPEED_MODE_GAD);
		NewGadget . ng_GadgetID		= Counter;

		GadgetArray[Counter++] = Gadget = CreateGadget(CHECKBOX_KIND,Gadget,&NewGadget,
			SZ_Adjust,	TRUE,

			GTCB_Checked,	Config -> SerialConfig -> HighSpeed,
		TAG_DONE);

		NewGadget . ng_GadgetText	= LocaleString(MSG_SERIALPANEL_CHECK_CARRIER_GAD);
		NewGadget . ng_GadgetID		= Counter;

		GadgetArray[Counter++] = Gadget = CreateGadget(CHECKBOX_KIND,Gadget,&NewGadget,
			SZ_Adjust,	TRUE,

			GTCB_Checked,	Config -> SerialConfig -> CheckCarrier,
		TAG_DONE);

		NewGadget . ng_GadgetText	= LocaleString(MSG_SERIALPANEL_SHARED_ACCESS_GAD);
		NewGadget . ng_GadgetID		= Counter;

		GadgetArray[Counter++] = Gadget = CreateGadget(CHECKBOX_KIND,Gadget,&NewGadget,
			SZ_Adjust,	TRUE,

			GTCB_Checked,	Config -> SerialConfig -> Shared,
		TAG_DONE);

		NewGadget . ng_GadgetText	= LocaleString(MSG_SERIALPANEL_XON_XOFF_GAD);
		NewGadget . ng_GadgetID		= Counter;

		GadgetArray[Counter++] = Gadget = CreateGadget(CHECKBOX_KIND,Gadget,&NewGadget,
			SZ_Adjust,	TRUE,

			GTCB_Checked,	Config -> SerialConfig -> xONxOFF,
		TAG_DONE);

		NewGadget . ng_GadgetText	= LocaleString(MSG_SERIALPANEL_PASS_XON_XOFF_THROUGH_GAD);
		NewGadget . ng_GadgetID		= Counter;

		GadgetArray[Counter++] = Gadget = CreateGadget(CHECKBOX_KIND,Gadget,&NewGadget,
			SZ_Adjust,	TRUE,

			GTCB_Checked,	Config -> SerialConfig -> PassThrough,
		TAG_DONE);

		NewGadget . ng_GadgetText	= LocaleString(MSG_SERIALPANEL_SERIAL_DEVICE_GAD);
		NewGadget . ng_GadgetID		= Counter;

		GadgetArray[Counter++] = Gadget = CreateGadget(STRING_KIND,Gadget,&NewGadget,
			SZ_Adjust,	TRUE,
			SZ_AutoWidth,	TRUE,
			SZ_LeaveFolder,	TRUE,

			GTST_MaxChars,	255,
			GTST_String,	Config -> SerialConfig -> SerialDevice,
		TAG_DONE);

		NewGadget . ng_GadgetID		= Counter;

		GadgetArray[Counter++] = Gadget = CreateGadget(FILE_KIND,Gadget,&NewGadget,
			SZ_Adjust,	TRUE,
			SZ_AutoWidth,	TRUE,
			SZ_PlaceLeft,	TRUE,
		TAG_DONE);

		NewGadget . ng_GadgetText	= LocaleString(MSG_SERIALPANEL_DEVICE_UNIT_NUMBER_GAD);
		NewGadget . ng_GadgetID		= Counter;

		GadgetArray[Counter++] = Gadget = CreateGadget(INTEGER_KIND,Gadget,&NewGadget,
			SZ_Adjust,	TRUE,
			SZ_AutoWidth,	TRUE,

			GTIN_MaxChars,	8,
			GTIN_Number,	Config -> SerialConfig -> UnitNumber,
		TAG_DONE);

		SZ_UpdateMaxWidth(BUTTON_KIND,LocaleString(MSG_GLOBAL_USE_GAD),0,NULL);
		SZ_UpdateMaxWidth(BUTTON_KIND,LocaleString(MSG_GLOBAL_CANCEL_GAD),0,NULL);
		SZ_UpdateMaxWidth(BUTTON_KIND,LocaleString(MSG_GLOBAL_DEFAULT_GAD),0,NULL);

		SZ_SetWidth(SZ_ResetMaxWidth());

		NewGadget . ng_GadgetText	= LocaleString(MSG_GLOBAL_USE_GAD);
		NewGadget . ng_GadgetID		= Counter;
		NewGadget . ng_Flags		= 0;

		GadgetArray[Counter++] = Gadget = CreateGadget(BUTTON_KIND,Gadget,&NewGadget,
			SZ_AlignExtra,	TRUE,
			SZ_Adjust,	TRUE,
			SZ_AutoWidth,	TRUE,
			SZ_AlignLeft,	TRUE,
			SZ_AlignBottom,	TRUE,
		TAG_DONE);

		NewGadget . ng_GadgetText	= LocaleString(MSG_GLOBAL_CANCEL_GAD);
		NewGadget . ng_GadgetID		= Counter;

		GadgetArray[Counter++] = Gadget = CreateGadget(BUTTON_KIND,Gadget,&NewGadget,
			SZ_Adjust,	TRUE,
			SZ_AutoWidth,	TRUE,
			SZ_AlignRight,	TRUE,
		TAG_DONE);

		if(Default)
		{
			NewGadget . ng_GadgetText	= LocaleString(MSG_GLOBAL_DEFAULT_GAD);
			NewGadget . ng_GadgetID		= Counter;

			GadgetArray[Counter] = Gadget = CreateGadget(BUTTON_KIND,Gadget,&NewGadget,
				SZ_Adjust,	TRUE,
				SZ_AutoWidth,	TRUE,
				SZ_AlignCentre,	TRUE,
			TAG_DONE);
		}
	}

	return(Gadget);
}

	/* SelectDevice(UBYTE *Name,struct Window *ParentWindow):
	 *
	 *	Select a new serial driver device using the
	 *	asl.library file requester.
	 */

STATIC BYTE
SelectDevice(UBYTE *Name,struct Window *ParentWindow)
{
	struct FileRequester	*AslFileRequest;
	UBYTE			*File;
	BYTE			 NewDevice = FALSE;

	File = Name;

	if(FilePart(File) == File)
		strcpy(SharedBuffer,"Devs:");
	else
	{
		UBYTE *Temp;

		strcpy(SharedBuffer,File);

		Temp = PathPart(SharedBuffer);

		Temp[0] = 0;

		File = FilePart(File);
	}

	if(AslFileRequest = (struct FileRequester *)AllocAslRequestTags(ASL_FileRequest,
		ASL_Window,	ParentWindow,
		ASL_File,	File,
		ASL_Dir,	SharedBuffer,
		ASL_Hail,	LocaleString(MSG_SERIALPANEL_SELECT_DEVICE_TXT),
		ASL_FuncFlags,	FILF_NEWIDCMP,
		ASL_Pattern,	"#?.device",
		ASL_OKText,	LocaleString(MSG_GLOBAL_SELECT_TXT),
	TAG_END))
	{
		if(AslRequestTags(AslFileRequest,TAG_DONE))
		{
			if(AslFileRequest -> rf_File[0])
			{
				if(Stricmp(AslFileRequest -> rf_Dir,"Devs:"))
				{
					strcpy(SharedBuffer,AslFileRequest -> rf_Dir);

					if(!AddPart(SharedBuffer,AslFileRequest -> rf_File,MAX_FILENAME_LENGTH))
						strcpy(SharedBuffer,AslFileRequest -> rf_File);
				}
				else
					strcpy(SharedBuffer,AslFileRequest -> rf_File);

				if(Stricmp(SharedBuffer,Name))
				{
					strcpy(Name,SharedBuffer);

					NewDevice = TRUE;
				}
			}
		}

		FreeAslRequest(AslFileRequest);
	}

	return(NewDevice);
}

BYTE
SerialPanel(struct Configuration *Config,BYTE *Default)
{
	struct Window	*PanelWindow;
	struct Gadget	*GadgetList = NULL;
	struct Gadget	*GadgetArray[GAD_DEFAULT + 1];
	UBYTE		 DummyBuffer[MAX_FILENAME_LENGTH];

	SaveConfig(Config,PrivateConfig);

	if(CreateAllGadgets(Config,GadgetArray,&GadgetList,Default))
	{
		if(PanelWindow = OpenWindowTags(NULL,
			WA_Left,	GetScreenLeft(Window) + (GetScreenWidth(Window) - SZ_GetWindowWidth()) / 2,
			WA_Top,		GetScreenTop(Window) + (GetScreenHeight(Window) - SZ_GetWindowHeight())	/ 2,
			WA_Width,	SZ_GetWindowWidth(),
			WA_Height,	SZ_GetWindowHeight(),

			WA_Activate,	TRUE,
			WA_DragBar,	TRUE,
			WA_DepthGadget,	TRUE,
			WA_RMBTrap,	TRUE,
			WA_DepthGadget,	TRUE,
			WA_CloseGadget,	TRUE,
			WA_CustomScreen,Window -> WScreen,

			WA_IDCMP,	IDCMP_GADGETDOWN | IDCMP_ACTIVEWINDOW | IDCMP_CLOSEWINDOW | IDCMP_RAWKEY | IDCMP_VANILLAKEY | SLIDERIDCMP | CHECKBOXIDCMP | BUTTONIDCMP | CYCLEIDCMP | INTEGERIDCMP | STRINGIDCMP,

			WA_Title,	LocaleString(MSG_SERIALPANEL_SERIAL_PREFERENCES_TXT),
		TAG_DONE))
		{
			struct IntuiMessage	*Massage;
			ULONG			 IClass,Code;
			struct Gadget		*Gadget;
			BYTE			 Terminated = FALSE;

			GuideContext(CONTEXT_SERIAL);

			PushWindow(PanelWindow);

			AddGList(PanelWindow,GadgetList,(UWORD)-1,(UWORD)-1,NULL);
			RefreshGList(GadgetList,PanelWindow,NULL,(UWORD)-1);
			GT_RefreshWindow(PanelWindow,NULL);

			ActiveGadget = NULL;

			SetSignal(0,SIG_BREAK);

			while(!Terminated)
			{
				if(Wait(PORTMASK(PanelWindow -> UserPort) | SIG_BREAK) & SIG_BREAK)
				{
					SaveConfig(PrivateConfig,Config);

					break;
				}

				while(!Terminated && (Massage = (struct IntuiMessage *)GT_GetIMsg(PanelWindow -> UserPort)))
				{
					IClass	= Massage -> Class;
					Code	= Massage -> Code;
					Gadget	= (struct Gadget *)Massage -> IAddress;

					GT_ReplyIMsg(Massage);

					KeySelect(GadgetArray,GAD_CANCEL,Code,PanelWindow,&Gadget,&IClass,&Code);

					if(IClass == IDCMP_GADGETDOWN)
					{
						if((Gadget -> GadgetType & GTYP_GTYPEMASK) == GTYP_STRGADGET)
							ActiveGadget = Gadget;
					}

					if(IClass == IDCMP_ACTIVEWINDOW && ActiveGadget)
						ActivateGadget(ActiveGadget,PanelWindow,NULL);

					if(IClass == IDCMP_CLOSEWINDOW)
					{
						SaveConfig(PrivateConfig,Config);

						Terminated = TRUE;
					}

					if(IClass == IDCMP_MOUSEMOVE)
					{
						switch(Gadget -> GadgetID)
						{
							case GAD_BAUD:

								Config -> SerialConfig -> BaudRate = BaudRates[Code];
								break;

							case GAD_BUFFERSIZE:

								Config -> SerialConfig -> SerialBufferSize = Sizes[Code];
								break;
						}
					}

					if(IClass == IDCMP_GADGETUP)
					{
						switch(Gadget -> GadgetID)
						{
							case GAD_HIGHSPEED:

								if(SZ_GetChecked(Gadget))
								{
									GT_SetGadgetAttrs(GadgetArray[GAD_DATABITS],PanelWindow,NULL,
										GA_Disabled,	TRUE,
										GTCY_Active,	1,
									TAG_DONE);

									Config -> SerialConfig -> BitsPerChar = 8;

									GT_SetGadgetAttrs(GadgetArray[GAD_STOPBITS],PanelWindow,NULL,
										GA_Disabled,	TRUE,
										GTCY_Active,	0,
									TAG_DONE);

									Config -> SerialConfig -> StopBits = 1;

									GT_SetGadgetAttrs(GadgetArray[GAD_PARITY],PanelWindow,NULL,
										GA_Disabled,	TRUE,
										GTCY_Active,	PARITY_NONE,
									TAG_DONE);

									Config -> SerialConfig -> Parity = PARITY_NONE;
								}
								else
								{
									GT_SetGadgetAttrs(GadgetArray[GAD_DATABITS],PanelWindow,NULL,
										GA_Disabled,FALSE,
									TAG_DONE);

									GT_SetGadgetAttrs(GadgetArray[GAD_STOPBITS],PanelWindow,NULL,
										GA_Disabled,FALSE,
									TAG_DONE);

									GT_SetGadgetAttrs(GadgetArray[GAD_PARITY],PanelWindow,NULL,
										GA_Disabled,FALSE,
									TAG_DONE);
								}

								break;

							case GAD_DUPLEX:

								Config -> SerialConfig -> Duplex = Code;
								break;

							case GAD_HANDSHAKING:

								Config -> SerialConfig -> HandshakingProtocol = Code;
								break;

							case GAD_STOPBITS:

								Config -> SerialConfig -> StopBits = 1 + Code;
								break;

							case GAD_PARITY:

								Config -> SerialConfig -> Parity = Code;
								break;

							case GAD_DATABITS:

								Config -> SerialConfig -> BitsPerChar = 7 + Code;
								break;

							case GAD_SELECT:

								BlockWindow(PanelWindow);

								strcpy(DummyBuffer,SZ_GetString(GadgetArray[GAD_DEVICE]));

								if(SelectDevice(DummyBuffer,PanelWindow))
								{
									GT_SetGadgetAttrs(GadgetArray[GAD_DEVICE],PanelWindow,NULL,
										GTST_String,DummyBuffer,
									TAG_DONE);
								}

								ReleaseWindow(PanelWindow);

								break;

							case GAD_USE:

								Terminated = TRUE;

								strcpy(Config -> SerialConfig -> SerialDevice,SZ_GetString(GadgetArray[GAD_DEVICE]));

								Config -> SerialConfig -> UnitNumber	= SZ_GetInteger(GadgetArray[GAD_UNIT]);

								Config -> SerialConfig -> BreakLength	= SZ_GetInteger(GadgetArray[GAD_BREAKLENGTH]);
								Config -> SerialConfig -> StripBit8	= SZ_GetChecked(GadgetArray[GAD_STRIP]);
								Config -> SerialConfig -> HighSpeed	= SZ_GetChecked(GadgetArray[GAD_HIGHSPEED]);
								Config -> SerialConfig -> CheckCarrier	= SZ_GetChecked(GadgetArray[GAD_CHECK_CARRIER]);
								Config -> SerialConfig -> Shared	= SZ_GetChecked(GadgetArray[GAD_SHARED]);
								Config -> SerialConfig -> xONxOFF	= SZ_GetChecked(GadgetArray[GAD_XON_XOFF]);
								Config -> SerialConfig -> PassThrough	= SZ_GetChecked(GadgetArray[GAD_PASSTHROUGH]);

								break;

							case GAD_CANCEL:

								Terminated = TRUE;

								SaveConfig(PrivateConfig,Config);

								break;

							case GAD_DEFAULT:

								*Default	= TRUE;

								Terminated	= TRUE;

								break;
						}
					}
				}
			}

			RemoveGList(PanelWindow,GadgetList,(UWORD)-1);

			PopWindow();

			CloseWindow(PanelWindow);
		}
	}

	FreeGadgets(GadgetList);

	return((BYTE)memcmp(PrivateConfig -> SerialConfig,Config -> SerialConfig,sizeof(struct SerialSettings)));
}
